# Copyright 2019 Atalaya Tech, Inc.

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

# http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import json
import logging
from typing import NamedTuple, Tuple, Iterator, Dict

import docker

from bentoml.exceptions import BentoMLException, MissingDependencyException

UNARY = 'UNARY'
SERVER_STREAMING = 'SERVER_STREAMING'
CLIENT_STREAMING = 'CLIENT_STREAMING'
BIDI_STREAMING = 'BIDI_STREAMING'
UNKNOWN = 'UNKNOWN'


logger = logging.getLogger(__name__)


def ensure_node_available_or_raise():
    from subprocess import CalledProcessError, check_output

    try:
        check_output(['node', '--version'])
    except CalledProcessError as error:
        raise BentoMLException(
            'Error executing node command: {}'.format(error.output.decode())
        )
    except FileNotFoundError:
        raise MissingDependencyException(
            'Node is required for Yatai web UI. Please visit '
            'www.nodejs.org for instructions'
        )


def parse_grpc_url(url):
    '''
    >>> parse_grpc_url("grpcs://yatai.com:43/query")
    ('grpcs', 'yatai.com:43/query')
    >>> parse_grpc_url("yatai.com:43/query")
    (None, 'yatai.com:43/query')
    '''
    from urllib3.util import parse_url

    parts = parse_url(url)
    return parts.scheme, url.replace(f"{parts.scheme}://", "", 1)


def wrap_interator_inc_counter(
    iterator, counter, grpc_type, grpc_service_name, grpc_method_name,
):
    for item in iterator:
        counter.labels(
            grpc_type=grpc_type,
            grpc_service=grpc_service_name,
            grpc_method=grpc_method_name,
        ).inc()
        yield item


# get method name for given RPC handler
def get_method_type(request_streaming, response_streaming) -> str:
    if not request_streaming and not response_streaming:
        return UNARY
    elif not request_streaming and response_streaming:
        return SERVER_STREAMING
    elif request_streaming and not response_streaming:
        return CLIENT_STREAMING
    elif request_streaming and response_streaming:
        return BIDI_STREAMING
    else:
        raise RuntimeError('Unknown request_streaming or response_streaming')


class MethodName(NamedTuple):
    """
    Represents a gRPC method name
    Attributes:
        package: This is defined by `package foo.bar`,
        designation in the protocol buffer definition
        service: service name in protocol buffer
        definition (eg: service SearchService { ... })
        method: method name
    """

    package: str
    service: str
    method: str

    @property
    def fully_qualified_service(self):
        """return the service name prefixed with package"""
        return f"{self.package}.{self.service}" if self.package else self.service


def parse_method_name(method_name: str) -> Tuple[MethodName, bool]:
    '''
    Infers the grpc service and method name from the handler_call_details.
    e.g. /package.ServiceName/MethodName
    '''
    parts = method_name.split("/")
    if len(parts) < 3:
        return MethodName("", "", ""), False
    _, package_service, method = parts
    *packages, service = package_service.rsplit(".", maxsplit=1)
    package = packages[0] if packages else ""
    return MethodName(package, service, method), True


def docker_build_logs(resp: Iterator):
    """
    Stream build logs to stderr.

    Args:
        resp (:obj:`Iterator`):
            blocking generator from docker.api.build

    Raises:
        docker.errors.BuildErrors:
            When errors occurs during build process. Usually
            this comes when generated Dockerfile are incorrect.
    """
    output: str = ""
    try:
        while True:
            try:
                # output logs to stdout
                # https://docker-py.readthedocs.io/en/stable/user_guides/multiplex.html
                output = next(resp).decode('utf-8')
                json_output: Dict = json.loads(output.strip('\r\n'))
                # output to stderr when running in docker
                if 'stream' in json_output:
                    logger.debug(json_output['stream'])
            except StopIteration:
                break
            except ValueError:
                logger.debug(f"Errors while building image:\n{output}")
    except docker.errors.BuildError as e:
        print(f"Failed to build container :\n{e.msg}")
        for line in e.build_log:
            if 'stream' in line:
                logger.debug(line['stream'].strip())



# Copyright 2019 Atalaya Tech, Inc.

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

# http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# List of APIs for accessing remote or local yatai service via Python

import io
import logging
import os
import shutil
import tarfile
import uuid
from typing import TYPE_CHECKING, Optional, Dict, List

import click
import grpc
import requests

from bentoml.exceptions import BentoMLException, BentoMLRpcError
from bentoml.saved_bundle import (
    save_to_dir,
    load_bento_service_metadata,
    safe_retrieve,
    load_from_dir,
)
from bentoml.utils import (
    status_pb_to_error_code_and_message,
    resolve_bento_bundle_uri,
    is_s3_url,
    is_gcs_url,
)
from bentoml.utils.lazy_loader import LazyLoader
from bentoml.utils.tempdir import TempDirectory
from bentoml.yatai.client.label_utils import generate_gprc_labels_selector
from bentoml.yatai.grpc_stream_utils import UploadBentoStreamRequests
from bentoml.yatai.proto import status_pb2
from bentoml.yatai.proto.repository_pb2 import (
    AddBentoRequest,
    GetBentoRequest,
    BentoUri,
    UpdateBentoRequest,
    UploadStatus,
    ListBentoRequest,
    DangerouslyDeleteBentoRequest,
    ContainerizeBentoRequest,
    DownloadBentoRequest,
)
from bentoml.yatai.proto.yatai_service_pb2_grpc import YataiStub
from bentoml.yatai.status import Status

if TYPE_CHECKING:
    from bentoml.service import BentoService
    from bentoml.yatai.client import YataiClient
    from bentoml.yatai.proto.repository_pb2 import Bento

logger = logging.getLogger(__name__)
yatai_proto = LazyLoader('yatai_proto', globals(), 'bentoml.yatai.proto')

# Default timeout in seconds per grpc request.
DEFAULT_GRPC_REQUEST_TIMEOUT = 6


def is_remote_yatai(yatai_service) -> bool:
    return isinstance(yatai_service, YataiStub)


class BentoRepositoryAPIClient:
    """
    Provided API to manage :class:`~bentoml.BentoService`
    for local Yatai repository
    """

    def __init__(self, yatai_service):
        # YataiService stub for accessing remote YataiService RPCs
        self.yatai_service: "YataiStub" = yatai_service

    def push(self, bento: str, with_labels: bool = True) -> "BentoUri":
        """
        Push a local BentoService to a remote yatai server.

        Args:
            bento (`str`):
                A BentoService identifier in the format of ``NAME:VERSION``.

            with_labels (`bool`, `optional`):
                Whether to push BentoService with given label.

        Returns: URI as gRPC stub for BentoService saved location.

        Example::

            from bentoml.yatai.client import get_yatai_client
            svc = MyBentoService()
            svc.save()

            remote_yatai_client = get_yatai_client('http://remote.yatai.service:50050')
            bento_name = f'{svc.name}:{svc.version}'
            remote_saved_path= remote_yatai_client.repository.push(bento_name)
        """
        from bentoml.yatai.client import get_yatai_client

        local_yc: "YataiClient" = get_yatai_client()

        local_bento_pb: "Bento" = local_yc.repository.get(bento)
        if local_bento_pb.uri.s3_presigned_url:
            bento_bundle_path = local_bento_pb.uri.s3_presigned_url
        elif local_bento_pb.uri.gcs_presigned_url:
            bento_bundle_path = local_bento_pb.uri.gcs_presigned_url
        else:
            bento_bundle_path = local_bento_pb.uri.uri
        labels = (
            dict(local_bento_pb.bento_service_metadata.labels)
            if with_labels is True and local_bento_pb.bento_service_metadata.labels
            else None
        )

        return self.upload_from_dir(bento_bundle_path, labels=labels)

    def pull(self, bento: str) -> "BentoUri":
        """
        Pull a :class:`~bentoml.BentoService` from remote Yatai.
        The BentoService will then be saved and registered to local Yatai.

        Args:
            bento (`str`):
                a BentoService identifier in the form of ``NAME:VERSION``

        Returns:
            :class:`reflection.GeneratedProtocolMessageType`:
                URI as gRPC stub for save location of BentoService.

        Example::

            from bentoml.yatai.client import get_yatai_client
            client = get_yatai_client('127.0.0.1:50051')
            saved_path = client.repository.pull('MyService:20210808_E38F3')
        """
        bento_pb: "Bento" = self.get(bento)
        with TempDirectory() as tmpdir:
            # Create a non-exist directory for safe_retrieve
            target_bundle_path = os.path.join(tmpdir, 'bundle')
            self.download_to_directory(bento_pb, target_bundle_path)

            from bentoml.yatai.client import get_yatai_client

            labels = (
                dict(bento_pb.bento_service_metadata.labels)
                if bento_pb.bento_service_metadata.labels
                else None
            )

            local_yc = get_yatai_client()
            return local_yc.repository.upload_from_dir(
                target_bundle_path, labels=labels
            )

    def upload(
        self, bento_service: "BentoService", version: str = None, labels: Dict = None,
    ) -> "BentoUri":
        """
        Save and upload given :class:`~bentoml.BentoService`
        to YataiService.

        Args:
            bento_service (:class:`~bentoml.BentoService`):
                a BentoService instance
            version (`str`, `optional`):
                version of ``bento_service``
            labels (`dict`, `optional`):
                :class:`~bentoml.BentoService` metadata

        Returns:
            BentoUri as gRPC stub for save location of BentoService.

        Example::

            from bentoml.yatai.client import get_yatai_client

            svc = MyBentoService()
            svc.save()

            remote_yatai_client = get_yatai_client('https://remote.yatai.service:50050')
            remote_path = remote_yatai_client.repository.upload(svc)
        """
        with TempDirectory() as tmpdir:
            save_to_dir(bento_service, tmpdir, version, silent=True)
            return self.upload_from_dir(tmpdir, labels)

    def upload_from_dir(self, saved_bento_path: str, labels: Dict = None) -> "BentoUri":
        from bentoml.yatai.db.stores.label import _validate_labels

        bento_service_metadata = load_bento_service_metadata(saved_bento_path)
        if labels:
            _validate_labels(labels)
            bento_service_metadata.labels.update(labels)

        get_bento_response = self.yatai_service.GetBento(
            GetBentoRequest(
                bento_name=bento_service_metadata.name,
                bento_version=bento_service_metadata.version,
            )
        )
        if get_bento_response.status.status_code == status_pb2.Status.OK:
            raise BentoMLException(
                "BentoService bundle {}:{} already registered in repository. Reset "
                "BentoService version with BentoService#set_version or bypass BentoML's"
                " model registry feature with BentoService#save_to_dir".format(
                    bento_service_metadata.name, bento_service_metadata.version
                )
            )
        elif get_bento_response.status.status_code != status_pb2.Status.NOT_FOUND:
            raise BentoMLException(
                'Failed accessing YataiService. {error_code}:'
                '{error_message}'.format(
                    error_code=Status.Name(get_bento_response.status.status_code),
                    error_message=get_bento_response.status.error_message,
                )
            )
        request = AddBentoRequest(
            bento_name=bento_service_metadata.name,
            bento_version=bento_service_metadata.version,
        )
        response = self.yatai_service.AddBento(request)

        if response.status.status_code != status_pb2.Status.OK:
            raise BentoMLException(
                "Error adding BentoService bundle to repository: {}:{}".format(
                    Status.Name(response.status.status_code),
                    response.status.error_message,
                )
            )

        if response.uri.type == BentoUri.LOCAL:
            # When using Yatai backed by File System repository,
            # if Yatai is a local instance, copy the files directly.
            # Otherwise, use UploadBento RPC to stream files to remote Yatai server
            if is_remote_yatai(self.yatai_service):
                self._upload_bento(
                    bento_service_metadata.name,
                    bento_service_metadata.version,
                    saved_bento_path,
                )
                update_bento_service = UpdateBentoRequest(
                    bento_name=bento_service_metadata.name,
                    bento_version=bento_service_metadata.version,
                    service_metadata=bento_service_metadata,
                )
                self.yatai_service.UpdateBento(update_bento_service)
            else:
                if os.path.exists(response.uri.uri):
                    raise BentoMLException(
                        f'Bento bundle directory {response.uri.uri} already exist'
                    )
                shutil.copytree(saved_bento_path, response.uri.uri)
                upload_status = UploadStatus.DONE

                self._update_bento_upload_progress(
                    bento_service_metadata, status=upload_status
                )

            logger.info(
                "BentoService bundle '%s:%s' saved to: %s",
                bento_service_metadata.name,
                bento_service_metadata.version,
                response.uri.uri,
            )
            # Return URI to saved bento in repository storage
            return response.uri.uri
        elif response.uri.type == BentoUri.S3 or response.uri.type == BentoUri.GCS:
            uri_type = 'S3' if response.uri.type == BentoUri.S3 else 'GCS'
            self._update_bento_upload_progress(
                bento_service_metadata, UploadStatus.UPLOADING, 0
            )

            fileobj = io.BytesIO()
            with tarfile.open(mode="w:gz", fileobj=fileobj) as tar:
                tar.add(saved_bento_path, arcname=bento_service_metadata.name)
            fileobj.seek(0, 0)

            if response.uri.type == BentoUri.S3:
                http_response = requests.put(
                    response.uri.s3_presigned_url, data=fileobj
                )
            elif response.uri.type == BentoUri.GCS:
                http_response = requests.put(
                    response.uri.gcs_presigned_url, data=fileobj
                )

            if http_response.status_code != 200:
                self._update_bento_upload_progress(
                    bento_service_metadata, UploadStatus.ERROR
                )
                raise BentoMLException(
                    f"Error saving BentoService bundle to {uri_type}."
                    f"{http_response.status_code}: {http_response.text}"
                )

            self._update_bento_upload_progress(bento_service_metadata)

            logger.info(
                "Successfully saved BentoService bundle '%s:%s' to {uri_type}: %s",
                bento_service_metadata.name,
                bento_service_metadata.version,
                response.uri.uri,
            )

            return response.uri.uri
        else:
            raise BentoMLException(
                f"Error saving Bento to target repository, URI type {response.uri.type}"
                f" at {response.uri.uri} not supported"
            )

    def _update_bento_upload_progress(
        self, bento_service_metadata, status=UploadStatus.DONE, percentage=None
    ):
        upload_status = UploadStatus(status=status, percentage=percentage)
        upload_status.updated_at.GetCurrentTime()
        update_bento_req = UpdateBentoRequest(
            bento_name=bento_service_metadata.name,
            bento_version=bento_service_metadata.version,
            upload_status=upload_status,
            service_metadata=bento_service_metadata,
        )
        self.yatai_service.UpdateBento(update_bento_req)

    def download_to_directory(self, bento_pb, target_dir: str) -> None:
        """
        Download or move bundle bundle to target directory.

        Args:
            bento_pb: bento bundle protobuf dict
            target_dir (`str`):

        Returns:
            None

        Raises:
            BentoMLException:
                Unrecognised Bento bundle storage type
        """
        if bento_pb.uri.type == BentoUri.S3:
            bento_service_bundle_path = bento_pb.uri.s3_presigned_url
        elif bento_pb.uri.type == BentoUri.GCS:
            bento_service_bundle_path = bento_pb.uri.gcs_presigned_url
        elif bento_pb.uri.type == BentoUri.LOCAL:
            # Download from remote yatai otherwise provide the file path.
            if is_remote_yatai(self.yatai_service):
                bento_service_bundle_path = self._download_bento(
                    bento_pb.name, bento_pb.version
                )
            else:
                bento_service_bundle_path = bento_pb.uri.uri
        else:
            raise BentoMLException(
                f'Unrecognized Bento bundle storage type {bento_pb.uri.type}'
            )

        safe_retrieve(bento_service_bundle_path, target_dir)

    def get(self, bento: str) -> "Bento":
        """
        Args:
            bento (`str`):
                A BentoService identifier in the format of ``NAME:VERSION``

        Returns:
            :class:`~bentoml.BentoService` metadata from Yatai RPC server.

        Raises:
            BentoMLException: ``bento`` is missing or have invalid format.

        Example::

            from bentoml.yatai.client import get_yatai_client
            yatai_client = get_yatai_client()
            bento_info = yatai_client.repository.get('my_service:version')
        """
        if ':' not in bento:
            raise BentoMLException(
                'BentoService name or version is missing. Please provide in the '
                'format of name:version'
            )
        name, version = bento.split(':')
        result = self.yatai_service.GetBento(
            GetBentoRequest(bento_name=name, bento_version=version)
        )
        if result.status.status_code != yatai_proto.status_pb2.Status.OK:
            error_code, error_message = status_pb_to_error_code_and_message(
                result.status
            )
            raise BentoMLException(
                f'BentoService {name}:{version} not found - '
                f'{error_code}:{error_message}'
            )
        return result.bento

    def list(
        self,
        bento_name: str = None,
        offset: int = None,
        limit: int = None,
        order_by: str = None,
        ascending_order: bool = None,
        labels: str = None,
    ) -> List["Bento"]:
        """
        List BentoServices that satisfy the specified criteria.

        Args:
            bento_name (`str`):
                BentoService name
            offset (`int`):
                offset of results
            limit (`int`):
                maximum number of returned results
            labels (`str`):
                sorted by given labels
            order_by (`str`):
                orders retrieved BentoService by :obj:`created_at` or :obj:`name`
            ascending_order (`bool`):
                direction of results order

        Returns:
            lists of :class:`~bentoml.BentoService` metadata.

        Example::

            from bentoml.yatai.client import get_yatai_client
            yatai_client = get_yatai_client()
            bentos_info_list = yatai_client.repository.list(labels='key=value,key2=value')
        """  # noqa: E501

        # TODO: ignore type checking for this function. This is
        #  due to all given arguments in `ListBentoRequest` are
        #  not optional types. One solution is to make all
        #  `ListBentoRequest` args in `list` positional. This could
        #  introduce different behaviour at different places in the
        #  codebase. Low triage
        list_bento_request = ListBentoRequest(
            bento_name=bento_name,  # type: ignore
            offset=offset,  # type: ignore
            limit=limit,  # type: ignore
            order_by=order_by,  # type: ignore
            ascending_order=ascending_order,  # type: ignore
        )

        if labels is not None:
            generate_gprc_labels_selector(list_bento_request.label_selectors, labels)

        result = self.yatai_service.ListBento(list_bento_request)
        if result.status.status_code != yatai_proto.status_pb2.Status.OK:
            error_code, error_message = status_pb_to_error_code_and_message(
                result.status
            )
            raise BentoMLException(f'{error_code}:{error_message}')
        return result.bentos

    def _delete_bento_bundle(self, bento_tag, require_confirm):
        bento_pb = self.get(bento_tag)
        if require_confirm and not click.confirm(f'Permanently delete {bento_tag}?'):
            return
        result = self.yatai_service.DangerouslyDeleteBento(
            DangerouslyDeleteBentoRequest(
                bento_name=bento_pb.name, bento_version=bento_pb.version
            )
        )

        if result.status.status_code != yatai_proto.status_pb2.Status.OK:
            error_code, error_message = status_pb_to_error_code_and_message(
                result.status
            )
            # Rather than raise Exception, continue to delete the next bentos
            logger.error(
                f'Failed to delete {bento_pb.name}:{bento_pb.version} - '
                f'{error_code}:{error_message}'
            )
        else:
            logger.info(f'Deleted {bento_pb.name}:{bento_pb.version}')

    def delete(
        self,
        bento_tag: str = None,
        labels: str = None,
        bento_name: str = None,
        bento_version: str = None,
        prune: Optional[bool] = False,  # pylint: disable=redefined-builtin
        require_confirm: Optional[bool] = False,
    ) -> None:
        """
        Delete bentos that matches the specified criteria.

        Args:
            bento_tag (`str`, `optional`):
                BentoService tags
            labels (`str`, `optional`):
                :class:`~bentoml.BentoService` labels
            bento_name (`str`, `optional`):
                given name of BentoService
            bento_version (`str`, `optional`):
                versions of given BentoService
            prune (`bool`, `optional`):
                Whether to delete all BentoService, default to ``False``
            require_confirm (`bool`, `optional`):
                Requires to confirm delete, default to ``False``

        Raises:
            BentoMLException: ``bento_tag``, ``bento_name`` and ``bento_version`` are parsed
                                at the same time

        Example::

            from bentoml.yatai.client import get_yatai_client

            yatai_client = get_yatai_client()
            # Delete all bento services
            yatai_client.repository.delete(prune=True)
            # Delete bento service with name is `IrisClassifier` and version `0.1.0`
            yatai_client.repository.delete(
                bento_name='IrisClassifier', bento_version='0.1.0'
            )
            # or use bento tag
            yatai_client.repository.delete('IrisClassifier:v0.1.0')
            # Delete all bento services with name 'MyService`
            yatai_client.repository.delete(bento_name='MyService')
            # Delete all bento services with labels match `ci=failed` and `cohort=20`
            yatai_client.repository.delete(labels='ci=failed, cohort=20')
        """  # noqa: E501
        delete_list_limit = 50

        if (
            bento_tag is not None
            and bento_name is not None
            and bento_version is not None
        ):
            raise BentoMLException('Too much arguments')

        if bento_tag is not None:
            logger.info(f'Deleting saved Bento bundle {bento_tag}')
            return self._delete_bento_bundle(bento_tag, require_confirm)
        elif bento_name is not None and bento_tag is not None:
            logger.info(f'Deleting saved Bento bundle {bento_name}:{bento_version}')
            return self._delete_bento_bundle(
                f'{bento_name}:{bento_version}', require_confirm
            )
        else:
            # list of bentos
            if prune is True:
                logger.info('Deleting all BentoML saved bundles.')
                # ignore other fields
                bento_name = None
                labels = None
            else:
                log_message = 'Deleting saved Bento bundles'
                if bento_name is not None:
                    log_message += f' with name: {bento_name},'
                if labels is not None:
                    log_message += f' with labels match to {labels}'
                logger.info(log_message)
            offset = 0
            while offset >= 0:
                bento_list = self.list(
                    bento_name=bento_name,
                    labels=labels,
                    offset=offset,
                    limit=delete_list_limit,
                )
                offset += delete_list_limit
                # Stop the loop, when no more bentos
                if len(bento_list) == 0:
                    break
                else:
                    for bento in bento_list:
                        self._delete_bento_bundle(
                            f'{bento.name}:{bento.version}', require_confirm
                        )

    def containerize(
        self,
        bento: str,
        tag: str,
        build_args: Dict[str, str] = None,
        push: bool = False,
    ) -> str:
        """
        Create a container image from a :class:`~bentoml.BentoService`

        Args:
            bento (`str`):
                A BentoService identifier with ``NAME:VERSION`` format.
            tag (`str`):
                BentoService tag.
            build_args (`Dict[str, str]`, `optional`):
                Build args to parse to ``docker build``
            push (`bool`, `optional`):
                Whether to push built container to remote YataiService.

        Returns:
            :obj:`str` representing the image tag of the containerized :class:`~bentoml.BentoService`

        Raises:
            BentoMLException: ``bento`` is missing or incorrect format.

        Example::

            svc = MyModelService()
            svc.save()

            from bentoml.yatai.client import get_yatai_client

            yatai_client = get_yatai_client()
            tag = yatai_client.repository.containerize(f'{svc.name}:{svc.version}')
        """  # noqa: E501
        if ':' not in bento:
            raise BentoMLException(
                'BentoService name or version is missing. Please provide in the '
                'format of name:version'
            )
        name, version = bento.split(':')
        containerize_request = ContainerizeBentoRequest(
            bento_name=name,
            bento_version=version,
            tag=tag,
            build_args=build_args,
            push=push,
        )
        result = self.yatai_service.ContainerizeBento(containerize_request)

        if result.status.status_code != yatai_proto.status_pb2.Status.OK:
            error_code, error_message = status_pb_to_error_code_and_message(
                result.status
            )
            raise BentoMLException(
                f'Failed to containerize {bento} - {error_code}:{error_message}'
            )
        return result.tag

    def load(self, bento: str) -> "BentoService":
        """
        Load :class:`~bentoml.BentoService` from nametag identifier
        or from a bento bundle path.

        Args:
            bento (`str`):
                :class:`~bentoml.BentoService` identifier or bundle path. Note
                that nametag identifier will have the following format: :code:`NAME:VERSION`

        Returns:
            :class:`~bentoml.BentoService`

        Example::

            from bentoml.yatai.client import get_yatai_client
            yatai_client = get_yatai_client()

            # Load BentoService bases on bento tag.
            bento_from_name = yatai_client.repository.load('service_name:version')
            # Load BentoService from bento bundle path
            bento_from_path = yatai_client.repository.load('/path/to/bento/bundle')
            # Load BentoService from s3 storage
            bento_from_reg = yatai_client.repository.load('s3://bucket/path/bundle.tar.gz')
        """  # noqa: E501
        if os.path.isdir(bento) or is_s3_url(bento) or is_gcs_url(bento):
            saved_bundle_path = bento
        else:
            bento_pb = self.get(bento)
            if bento_pb.uri.type == BentoUri.LOCAL and is_remote_yatai(
                self.yatai_service
            ):
                saved_bundle_path = self._download_bento(
                    bento_pb.name, bento_pb.version
                )
            else:
                saved_bundle_path = resolve_bento_bundle_uri(bento_pb)
        svc = load_from_dir(saved_bundle_path)
        return svc

    def _upload_bento(self, bento_name, bento_version, saved_bento_bundle_path):
        try:
            streaming_request_generator = UploadBentoStreamRequests(
                bento_name=bento_name,
                bento_version=bento_version,
                bento_bundle_path=saved_bento_bundle_path,
            )
            result = self.yatai_service.UploadBento(
                iter(streaming_request_generator,),
                timeout=DEFAULT_GRPC_REQUEST_TIMEOUT,
            )
            if result.status.status_code != status_pb2.Status.OK:
                raise BentoMLException(result.status.error_message)
        except grpc.RpcError as e:
            raise BentoMLRpcError(
                e,
                f'Failed to upload {bento_name}:{bento_version} to '
                f'the remote yatai server',
            )
        finally:
            streaming_request_generator.close()

    def _download_bento(self, bento_name, bento_version):
        with TempDirectory(cleanup=False) as temp_dir:
            try:
                temp_tar_path = os.path.join(temp_dir, f'{uuid.uuid4().hex[:12]}.tar')
                response_iterator = self.yatai_service.DownloadBento(
                    DownloadBentoRequest(
                        bento_name=bento_name, bento_version=bento_version
                    ),
                    timeout=DEFAULT_GRPC_REQUEST_TIMEOUT,
                )
                with open(temp_tar_path, 'wb+') as file:
                    for response in response_iterator:
                        if response.status.status_code != status_pb2.Status.OK:
                            raise BentoMLException(response.status.error_message)
                        file.write(response.bento_bundle)
                    file.seek(0)
                    temp_bundle_path = os.path.join(
                        temp_dir, f'{bento_name}_{bento_version}'
                    )
                    with tarfile.open(fileobj=file, mode='r') as tar:
                        tar.extractall(path=temp_bundle_path)
                return temp_bundle_path
            except grpc.RpcError as e:
                raise BentoMLRpcError(
                    e,
                    f'Failed to download {bento_name}:{bento_version} from '
                    f'the remote yatai server',
                )



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: repository.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import struct_pb2 as google_dot_protobuf_dot_struct__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
import bentoml.yatai.proto.status_pb2 as status__pb2
import bentoml.yatai.proto.label_selectors_pb2 as label__selectors__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='repository.proto',
  package='bentoml',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x10repository.proto\x12\x07\x62\x65ntoml\x1a\x1cgoogle/protobuf/struct.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x0cstatus.proto\x1a\x15label_selectors.proto\"\xd1\x01\n\x08\x42\x65ntoUri\x12+\n\x04type\x18\x01 \x01(\x0e\x32\x1d.bentoml.BentoUri.StorageType\x12\x0b\n\x03uri\x18\x02 \x01(\t\x12\x18\n\x10s3_presigned_url\x18\x03 \x01(\t\x12\x19\n\x11gcs_presigned_url\x18\x04 \x01(\t\"V\n\x0bStorageType\x12\t\n\x05UNSET\x10\x00\x12\t\n\x05LOCAL\x10\x01\x12\x06\n\x02S3\x10\x02\x12\x07\n\x03GCS\x10\x03\x12\x16\n\x12\x41ZURE_BLOB_STORAGE\x10\x04\x12\x08\n\x04HDFS\x10\x05\"\xa8\x07\n\x14\x42\x65ntoServiceMetadata\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0f\n\x07version\x18\x02 \x01(\t\x12.\n\ncreated_at\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12:\n\x03\x65nv\x18\x04 \x01(\x0b\x32-.bentoml.BentoServiceMetadata.BentoServiceEnv\x12>\n\tartifacts\x18\x05 \x03(\x0b\x32+.bentoml.BentoServiceMetadata.BentoArtifact\x12;\n\x04\x61pis\x18\x06 \x03(\x0b\x32-.bentoml.BentoServiceMetadata.BentoServiceApi\x12\x39\n\x06labels\x18\x07 \x03(\x0b\x32).bentoml.BentoServiceMetadata.LabelsEntry\x1a\xb3\x01\n\x0f\x42\x65ntoServiceEnv\x12\x10\n\x08setup_sh\x18\x01 \x01(\t\x12\x11\n\tconda_env\x18\x02 \x01(\t\x12\x18\n\x10pip_dependencies\x18\x03 \x01(\t\x12\x16\n\x0epython_version\x18\x04 \x01(\t\x12\x19\n\x11\x64ocker_base_image\x18\x05 \x01(\t\x12\x14\n\x0cpip_packages\x18\x06 \x03(\t\x12\x18\n\x10requirements_txt\x18\x07 \x01(\t\x1a_\n\rBentoArtifact\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x15\n\rartifact_type\x18\x02 \x01(\t\x12)\n\x08metadata\x18\x03 \x01(\x0b\x32\x17.google.protobuf.Struct\x1a\x86\x02\n\x0f\x42\x65ntoServiceApi\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x12\n\ninput_type\x18\x02 \x01(\t\x12\x0c\n\x04\x64ocs\x18\x03 \x01(\t\x12-\n\x0cinput_config\x18\x04 \x01(\x0b\x32\x17.google.protobuf.Struct\x12.\n\routput_config\x18\x05 \x01(\x0b\x32\x17.google.protobuf.Struct\x12\x13\n\x0boutput_type\x18\x06 \x01(\t\x12\x16\n\x0emb_max_latency\x18\x07 \x01(\x05\x12\x19\n\x11mb_max_batch_size\x18\x08 \x01(\x05\x12\r\n\x05\x62\x61tch\x18\t \x01(\x08\x12\r\n\x05route\x18\n \x01(\t\x1a-\n\x0bLabelsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\xac\x01\n\x05\x42\x65nto\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0f\n\x07version\x18\x02 \x01(\t\x12\x1e\n\x03uri\x18\x03 \x01(\x0b\x32\x11.bentoml.BentoUri\x12=\n\x16\x62\x65nto_service_metadata\x18\x04 \x01(\x0b\x32\x1d.bentoml.BentoServiceMetadata\x12%\n\x06status\x18\x05 \x01(\x0b\x32\x15.bentoml.UploadStatus\"<\n\x0f\x41\x64\x64\x42\x65ntoRequest\x12\x12\n\nbento_name\x18\x01 \x01(\t\x12\x15\n\rbento_version\x18\x02 \x01(\t\"S\n\x10\x41\x64\x64\x42\x65ntoResponse\x12\x1f\n\x06status\x18\x01 \x01(\x0b\x32\x0f.bentoml.Status\x12\x1e\n\x03uri\x18\x02 \x01(\x0b\x32\x11.bentoml.BentoUri\"\xe5\x01\n\x0cUploadStatus\x12,\n\x06status\x18\x01 \x01(\x0e\x32\x1c.bentoml.UploadStatus.Status\x12.\n\nupdated_at\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x12\n\npercentage\x18\x03 \x01(\x05\x12\x15\n\rerror_message\x18\x04 \x01(\t\"L\n\x06Status\x12\x11\n\rUNINITIALIZED\x10\x00\x12\r\n\tUPLOADING\x10\x01\x12\x08\n\x04\x44ONE\x10\x02\x12\t\n\x05\x45RROR\x10\x03\x12\x0b\n\x07TIMEOUT\x10\x04\"\xa6\x01\n\x12UpdateBentoRequest\x12\x12\n\nbento_name\x18\x01 \x01(\t\x12\x15\n\rbento_version\x18\x02 \x01(\t\x12,\n\rupload_status\x18\x03 \x01(\x0b\x32\x15.bentoml.UploadStatus\x12\x37\n\x10service_metadata\x18\x04 \x01(\x0b\x32\x1d.bentoml.BentoServiceMetadata\"6\n\x13UpdateBentoResponse\x12\x1f\n\x06status\x18\x01 \x01(\x0b\x32\x0f.bentoml.Status\"J\n\x1d\x44\x61ngerouslyDeleteBentoRequest\x12\x12\n\nbento_name\x18\x01 \x01(\t\x12\x15\n\rbento_version\x18\x02 \x01(\t\"A\n\x1e\x44\x61ngerouslyDeleteBentoResponse\x12\x1f\n\x06status\x18\x01 \x01(\x0b\x32\x0f.bentoml.Status\"<\n\x0fGetBentoRequest\x12\x12\n\nbento_name\x18\x01 \x01(\t\x12\x15\n\rbento_version\x18\x02 \x01(\t\"R\n\x10GetBentoResponse\x12\x1f\n\x06status\x18\x01 \x01(\x0b\x32\x0f.bentoml.Status\x12\x1d\n\x05\x62\x65nto\x18\x02 \x01(\x0b\x32\x0e.bentoml.Bento\"\xfa\x01\n\x10ListBentoRequest\x12\x12\n\nbento_name\x18\x01 \x01(\t\x12\x0e\n\x06offset\x18\x02 \x01(\x05\x12\r\n\x05limit\x18\x03 \x01(\x05\x12;\n\x08order_by\x18\x04 \x01(\x0e\x32).bentoml.ListBentoRequest.SORTABLE_COLUMN\x12\x17\n\x0f\x61scending_order\x18\x05 \x01(\x08\x12\x30\n\x0flabel_selectors\x18\x06 \x01(\x0b\x32\x17.bentoml.LabelSelectors\"+\n\x0fSORTABLE_COLUMN\x12\x0e\n\ncreated_at\x10\x00\x12\x08\n\x04name\x10\x01\"T\n\x11ListBentoResponse\x12\x1f\n\x06status\x18\x01 \x01(\x0b\x32\x0f.bentoml.Status\x12\x1e\n\x06\x62\x65ntos\x18\x02 \x03(\x0b\x32\x0e.bentoml.Bento\"\xd8\x01\n\x18\x43ontainerizeBentoRequest\x12\x12\n\nbento_name\x18\x01 \x01(\t\x12\x15\n\rbento_version\x18\x02 \x01(\t\x12\x0b\n\x03tag\x18\x03 \x01(\t\x12\x44\n\nbuild_args\x18\x04 \x03(\x0b\x32\x30.bentoml.ContainerizeBentoRequest.BuildArgsEntry\x12\x0c\n\x04push\x18\x05 \x01(\x08\x1a\x30\n\x0e\x42uildArgsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"I\n\x19\x43ontainerizeBentoResponse\x12\x1f\n\x06status\x18\x01 \x01(\x0b\x32\x0f.bentoml.Status\x12\x0b\n\x03tag\x18\x02 \x01(\t\"U\n\x12UploadBentoRequest\x12\x12\n\nbento_name\x18\x01 \x01(\t\x12\x15\n\rbento_version\x18\x02 \x01(\t\x12\x14\n\x0c\x62\x65nto_bundle\x18\x03 \x01(\x0c\"6\n\x13UploadBentoResponse\x12\x1f\n\x06status\x18\x01 \x01(\x0b\x32\x0f.bentoml.Status\"A\n\x14\x44ownloadBentoRequest\x12\x12\n\nbento_name\x18\x01 \x01(\t\x12\x15\n\rbento_version\x18\x02 \x01(\t\"N\n\x15\x44ownloadBentoResponse\x12\x14\n\x0c\x62\x65nto_bundle\x18\x01 \x01(\x0c\x12\x1f\n\x06status\x18\x02 \x01(\x0b\x32\x0f.bentoml.Statusb\x06proto3'
  ,
  dependencies=[google_dot_protobuf_dot_struct__pb2.DESCRIPTOR,google_dot_protobuf_dot_timestamp__pb2.DESCRIPTOR,status__pb2.DESCRIPTOR,label__selectors__pb2.DESCRIPTOR,])



_BENTOURI_STORAGETYPE = _descriptor.EnumDescriptor(
  name='StorageType',
  full_name='bentoml.BentoUri.StorageType',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='UNSET', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='LOCAL', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='S3', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='GCS', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='AZURE_BLOB_STORAGE', index=4, number=4,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='HDFS', index=5, number=5,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=253,
  serialized_end=339,
)
_sym_db.RegisterEnumDescriptor(_BENTOURI_STORAGETYPE)

_UPLOADSTATUS_STATUS = _descriptor.EnumDescriptor(
  name='Status',
  full_name='bentoml.UploadStatus.Status',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='UNINITIALIZED', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='UPLOADING', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='DONE', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ERROR', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='TIMEOUT', index=4, number=4,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1756,
  serialized_end=1832,
)
_sym_db.RegisterEnumDescriptor(_UPLOADSTATUS_STATUS)

_LISTBENTOREQUEST_SORTABLE_COLUMN = _descriptor.EnumDescriptor(
  name='SORTABLE_COLUMN',
  full_name='bentoml.ListBentoRequest.SORTABLE_COLUMN',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='created_at', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='name', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=2556,
  serialized_end=2599,
)
_sym_db.RegisterEnumDescriptor(_LISTBENTOREQUEST_SORTABLE_COLUMN)


_BENTOURI = _descriptor.Descriptor(
  name='BentoUri',
  full_name='bentoml.BentoUri',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='type', full_name='bentoml.BentoUri.type', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='uri', full_name='bentoml.BentoUri.uri', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='s3_presigned_url', full_name='bentoml.BentoUri.s3_presigned_url', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='gcs_presigned_url', full_name='bentoml.BentoUri.gcs_presigned_url', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _BENTOURI_STORAGETYPE,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=130,
  serialized_end=339,
)


_BENTOSERVICEMETADATA_BENTOSERVICEENV = _descriptor.Descriptor(
  name='BentoServiceEnv',
  full_name='bentoml.BentoServiceMetadata.BentoServiceEnv',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='setup_sh', full_name='bentoml.BentoServiceMetadata.BentoServiceEnv.setup_sh', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='conda_env', full_name='bentoml.BentoServiceMetadata.BentoServiceEnv.conda_env', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='pip_dependencies', full_name='bentoml.BentoServiceMetadata.BentoServiceEnv.pip_dependencies', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='python_version', full_name='bentoml.BentoServiceMetadata.BentoServiceEnv.python_version', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='docker_base_image', full_name='bentoml.BentoServiceMetadata.BentoServiceEnv.docker_base_image', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='pip_packages', full_name='bentoml.BentoServiceMetadata.BentoServiceEnv.pip_packages', index=5,
      number=6, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='requirements_txt', full_name='bentoml.BentoServiceMetadata.BentoServiceEnv.requirements_txt', index=6,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=690,
  serialized_end=869,
)

_BENTOSERVICEMETADATA_BENTOARTIFACT = _descriptor.Descriptor(
  name='BentoArtifact',
  full_name='bentoml.BentoServiceMetadata.BentoArtifact',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='bentoml.BentoServiceMetadata.BentoArtifact.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='artifact_type', full_name='bentoml.BentoServiceMetadata.BentoArtifact.artifact_type', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='metadata', full_name='bentoml.BentoServiceMetadata.BentoArtifact.metadata', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=871,
  serialized_end=966,
)

_BENTOSERVICEMETADATA_BENTOSERVICEAPI = _descriptor.Descriptor(
  name='BentoServiceApi',
  full_name='bentoml.BentoServiceMetadata.BentoServiceApi',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='bentoml.BentoServiceMetadata.BentoServiceApi.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='input_type', full_name='bentoml.BentoServiceMetadata.BentoServiceApi.input_type', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='docs', full_name='bentoml.BentoServiceMetadata.BentoServiceApi.docs', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='input_config', full_name='bentoml.BentoServiceMetadata.BentoServiceApi.input_config', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='output_config', full_name='bentoml.BentoServiceMetadata.BentoServiceApi.output_config', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='output_type', full_name='bentoml.BentoServiceMetadata.BentoServiceApi.output_type', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='mb_max_latency', full_name='bentoml.BentoServiceMetadata.BentoServiceApi.mb_max_latency', index=6,
      number=7, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='mb_max_batch_size', full_name='bentoml.BentoServiceMetadata.BentoServiceApi.mb_max_batch_size', index=7,
      number=8, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='batch', full_name='bentoml.BentoServiceMetadata.BentoServiceApi.batch', index=8,
      number=9, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='route', full_name='bentoml.BentoServiceMetadata.BentoServiceApi.route', index=9,
      number=10, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=969,
  serialized_end=1231,
)

_BENTOSERVICEMETADATA_LABELSENTRY = _descriptor.Descriptor(
  name='LabelsEntry',
  full_name='bentoml.BentoServiceMetadata.LabelsEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='bentoml.BentoServiceMetadata.LabelsEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='bentoml.BentoServiceMetadata.LabelsEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'8\001',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1233,
  serialized_end=1278,
)

_BENTOSERVICEMETADATA = _descriptor.Descriptor(
  name='BentoServiceMetadata',
  full_name='bentoml.BentoServiceMetadata',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='bentoml.BentoServiceMetadata.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='version', full_name='bentoml.BentoServiceMetadata.version', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='created_at', full_name='bentoml.BentoServiceMetadata.created_at', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='env', full_name='bentoml.BentoServiceMetadata.env', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='artifacts', full_name='bentoml.BentoServiceMetadata.artifacts', index=4,
      number=5, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='apis', full_name='bentoml.BentoServiceMetadata.apis', index=5,
      number=6, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='labels', full_name='bentoml.BentoServiceMetadata.labels', index=6,
      number=7, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_BENTOSERVICEMETADATA_BENTOSERVICEENV, _BENTOSERVICEMETADATA_BENTOARTIFACT, _BENTOSERVICEMETADATA_BENTOSERVICEAPI, _BENTOSERVICEMETADATA_LABELSENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=342,
  serialized_end=1278,
)


_BENTO = _descriptor.Descriptor(
  name='Bento',
  full_name='bentoml.Bento',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='bentoml.Bento.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='version', full_name='bentoml.Bento.version', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='uri', full_name='bentoml.Bento.uri', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='bento_service_metadata', full_name='bentoml.Bento.bento_service_metadata', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='status', full_name='bentoml.Bento.status', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1281,
  serialized_end=1453,
)


_ADDBENTOREQUEST = _descriptor.Descriptor(
  name='AddBentoRequest',
  full_name='bentoml.AddBentoRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='bento_name', full_name='bentoml.AddBentoRequest.bento_name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='bento_version', full_name='bentoml.AddBentoRequest.bento_version', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1455,
  serialized_end=1515,
)


_ADDBENTORESPONSE = _descriptor.Descriptor(
  name='AddBentoResponse',
  full_name='bentoml.AddBentoResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='status', full_name='bentoml.AddBentoResponse.status', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='uri', full_name='bentoml.AddBentoResponse.uri', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1517,
  serialized_end=1600,
)


_UPLOADSTATUS = _descriptor.Descriptor(
  name='UploadStatus',
  full_name='bentoml.UploadStatus',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='status', full_name='bentoml.UploadStatus.status', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='updated_at', full_name='bentoml.UploadStatus.updated_at', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='percentage', full_name='bentoml.UploadStatus.percentage', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='error_message', full_name='bentoml.UploadStatus.error_message', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _UPLOADSTATUS_STATUS,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1603,
  serialized_end=1832,
)


_UPDATEBENTOREQUEST = _descriptor.Descriptor(
  name='UpdateBentoRequest',
  full_name='bentoml.UpdateBentoRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='bento_name', full_name='bentoml.UpdateBentoRequest.bento_name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='bento_version', full_name='bentoml.UpdateBentoRequest.bento_version', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='upload_status', full_name='bentoml.UpdateBentoRequest.upload_status', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='service_metadata', full_name='bentoml.UpdateBentoRequest.service_metadata', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1835,
  serialized_end=2001,
)


_UPDATEBENTORESPONSE = _descriptor.Descriptor(
  name='UpdateBentoResponse',
  full_name='bentoml.UpdateBentoResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='status', full_name='bentoml.UpdateBentoResponse.status', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2003,
  serialized_end=2057,
)


_DANGEROUSLYDELETEBENTOREQUEST = _descriptor.Descriptor(
  name='DangerouslyDeleteBentoRequest',
  full_name='bentoml.DangerouslyDeleteBentoRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='bento_name', full_name='bentoml.DangerouslyDeleteBentoRequest.bento_name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='bento_version', full_name='bentoml.DangerouslyDeleteBentoRequest.bento_version', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2059,
  serialized_end=2133,
)


_DANGEROUSLYDELETEBENTORESPONSE = _descriptor.Descriptor(
  name='DangerouslyDeleteBentoResponse',
  full_name='bentoml.DangerouslyDeleteBentoResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='status', full_name='bentoml.DangerouslyDeleteBentoResponse.status', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2135,
  serialized_end=2200,
)


_GETBENTOREQUEST = _descriptor.Descriptor(
  name='GetBentoRequest',
  full_name='bentoml.GetBentoRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='bento_name', full_name='bentoml.GetBentoRequest.bento_name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='bento_version', full_name='bentoml.GetBentoRequest.bento_version', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2202,
  serialized_end=2262,
)


_GETBENTORESPONSE = _descriptor.Descriptor(
  name='GetBentoResponse',
  full_name='bentoml.GetBentoResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='status', full_name='bentoml.GetBentoResponse.status', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='bento', full_name='bentoml.GetBentoResponse.bento', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2264,
  serialized_end=2346,
)


_LISTBENTOREQUEST = _descriptor.Descriptor(
  name='ListBentoRequest',
  full_name='bentoml.ListBentoRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='bento_name', full_name='bentoml.ListBentoRequest.bento_name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='offset', full_name='bentoml.ListBentoRequest.offset', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='limit', full_name='bentoml.ListBentoRequest.limit', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='order_by', full_name='bentoml.ListBentoRequest.order_by', index=3,
      number=4, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='ascending_order', full_name='bentoml.ListBentoRequest.ascending_order', index=4,
      number=5, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='label_selectors', full_name='bentoml.ListBentoRequest.label_selectors', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _LISTBENTOREQUEST_SORTABLE_COLUMN,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2349,
  serialized_end=2599,
)


_LISTBENTORESPONSE = _descriptor.Descriptor(
  name='ListBentoResponse',
  full_name='bentoml.ListBentoResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='status', full_name='bentoml.ListBentoResponse.status', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='bentos', full_name='bentoml.ListBentoResponse.bentos', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2601,
  serialized_end=2685,
)


_CONTAINERIZEBENTOREQUEST_BUILDARGSENTRY = _descriptor.Descriptor(
  name='BuildArgsEntry',
  full_name='bentoml.ContainerizeBentoRequest.BuildArgsEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='bentoml.ContainerizeBentoRequest.BuildArgsEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='bentoml.ContainerizeBentoRequest.BuildArgsEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'8\001',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2856,
  serialized_end=2904,
)

_CONTAINERIZEBENTOREQUEST = _descriptor.Descriptor(
  name='ContainerizeBentoRequest',
  full_name='bentoml.ContainerizeBentoRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='bento_name', full_name='bentoml.ContainerizeBentoRequest.bento_name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='bento_version', full_name='bentoml.ContainerizeBentoRequest.bento_version', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='tag', full_name='bentoml.ContainerizeBentoRequest.tag', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='build_args', full_name='bentoml.ContainerizeBentoRequest.build_args', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='push', full_name='bentoml.ContainerizeBentoRequest.push', index=4,
      number=5, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_CONTAINERIZEBENTOREQUEST_BUILDARGSENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2688,
  serialized_end=2904,
)


_CONTAINERIZEBENTORESPONSE = _descriptor.Descriptor(
  name='ContainerizeBentoResponse',
  full_name='bentoml.ContainerizeBentoResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='status', full_name='bentoml.ContainerizeBentoResponse.status', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='tag', full_name='bentoml.ContainerizeBentoResponse.tag', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2906,
  serialized_end=2979,
)


_UPLOADBENTOREQUEST = _descriptor.Descriptor(
  name='UploadBentoRequest',
  full_name='bentoml.UploadBentoRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='bento_name', full_name='bentoml.UploadBentoRequest.bento_name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='bento_version', full_name='bentoml.UploadBentoRequest.bento_version', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='bento_bundle', full_name='bentoml.UploadBentoRequest.bento_bundle', index=2,
      number=3, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2981,
  serialized_end=3066,
)


_UPLOADBENTORESPONSE = _descriptor.Descriptor(
  name='UploadBentoResponse',
  full_name='bentoml.UploadBentoResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='status', full_name='bentoml.UploadBentoResponse.status', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3068,
  serialized_end=3122,
)


_DOWNLOADBENTOREQUEST = _descriptor.Descriptor(
  name='DownloadBentoRequest',
  full_name='bentoml.DownloadBentoRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='bento_name', full_name='bentoml.DownloadBentoRequest.bento_name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='bento_version', full_name='bentoml.DownloadBentoRequest.bento_version', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3124,
  serialized_end=3189,
)


_DOWNLOADBENTORESPONSE = _descriptor.Descriptor(
  name='DownloadBentoResponse',
  full_name='bentoml.DownloadBentoResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='bento_bundle', full_name='bentoml.DownloadBentoResponse.bento_bundle', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='status', full_name='bentoml.DownloadBentoResponse.status', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3191,
  serialized_end=3269,
)

_BENTOURI.fields_by_name['type'].enum_type = _BENTOURI_STORAGETYPE
_BENTOURI_STORAGETYPE.containing_type = _BENTOURI
_BENTOSERVICEMETADATA_BENTOSERVICEENV.containing_type = _BENTOSERVICEMETADATA
_BENTOSERVICEMETADATA_BENTOARTIFACT.fields_by_name['metadata'].message_type = google_dot_protobuf_dot_struct__pb2._STRUCT
_BENTOSERVICEMETADATA_BENTOARTIFACT.containing_type = _BENTOSERVICEMETADATA
_BENTOSERVICEMETADATA_BENTOSERVICEAPI.fields_by_name['input_config'].message_type = google_dot_protobuf_dot_struct__pb2._STRUCT
_BENTOSERVICEMETADATA_BENTOSERVICEAPI.fields_by_name['output_config'].message_type = google_dot_protobuf_dot_struct__pb2._STRUCT
_BENTOSERVICEMETADATA_BENTOSERVICEAPI.containing_type = _BENTOSERVICEMETADATA
_BENTOSERVICEMETADATA_LABELSENTRY.containing_type = _BENTOSERVICEMETADATA
_BENTOSERVICEMETADATA.fields_by_name['created_at'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_BENTOSERVICEMETADATA.fields_by_name['env'].message_type = _BENTOSERVICEMETADATA_BENTOSERVICEENV
_BENTOSERVICEMETADATA.fields_by_name['artifacts'].message_type = _BENTOSERVICEMETADATA_BENTOARTIFACT
_BENTOSERVICEMETADATA.fields_by_name['apis'].message_type = _BENTOSERVICEMETADATA_BENTOSERVICEAPI
_BENTOSERVICEMETADATA.fields_by_name['labels'].message_type = _BENTOSERVICEMETADATA_LABELSENTRY
_BENTO.fields_by_name['uri'].message_type = _BENTOURI
_BENTO.fields_by_name['bento_service_metadata'].message_type = _BENTOSERVICEMETADATA
_BENTO.fields_by_name['status'].message_type = _UPLOADSTATUS
_ADDBENTORESPONSE.fields_by_name['status'].message_type = status__pb2._STATUS
_ADDBENTORESPONSE.fields_by_name['uri'].message_type = _BENTOURI
_UPLOADSTATUS.fields_by_name['status'].enum_type = _UPLOADSTATUS_STATUS
_UPLOADSTATUS.fields_by_name['updated_at'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_UPLOADSTATUS_STATUS.containing_type = _UPLOADSTATUS
_UPDATEBENTOREQUEST.fields_by_name['upload_status'].message_type = _UPLOADSTATUS
_UPDATEBENTOREQUEST.fields_by_name['service_metadata'].message_type = _BENTOSERVICEMETADATA
_UPDATEBENTORESPONSE.fields_by_name['status'].message_type = status__pb2._STATUS
_DANGEROUSLYDELETEBENTORESPONSE.fields_by_name['status'].message_type = status__pb2._STATUS
_GETBENTORESPONSE.fields_by_name['status'].message_type = status__pb2._STATUS
_GETBENTORESPONSE.fields_by_name['bento'].message_type = _BENTO
_LISTBENTOREQUEST.fields_by_name['order_by'].enum_type = _LISTBENTOREQUEST_SORTABLE_COLUMN
_LISTBENTOREQUEST.fields_by_name['label_selectors'].message_type = label__selectors__pb2._LABELSELECTORS
_LISTBENTOREQUEST_SORTABLE_COLUMN.containing_type = _LISTBENTOREQUEST
_LISTBENTORESPONSE.fields_by_name['status'].message_type = status__pb2._STATUS
_LISTBENTORESPONSE.fields_by_name['bentos'].message_type = _BENTO
_CONTAINERIZEBENTOREQUEST_BUILDARGSENTRY.containing_type = _CONTAINERIZEBENTOREQUEST
_CONTAINERIZEBENTOREQUEST.fields_by_name['build_args'].message_type = _CONTAINERIZEBENTOREQUEST_BUILDARGSENTRY
_CONTAINERIZEBENTORESPONSE.fields_by_name['status'].message_type = status__pb2._STATUS
_UPLOADBENTORESPONSE.fields_by_name['status'].message_type = status__pb2._STATUS
_DOWNLOADBENTORESPONSE.fields_by_name['status'].message_type = status__pb2._STATUS
DESCRIPTOR.message_types_by_name['BentoUri'] = _BENTOURI
DESCRIPTOR.message_types_by_name['BentoServiceMetadata'] = _BENTOSERVICEMETADATA
DESCRIPTOR.message_types_by_name['Bento'] = _BENTO
DESCRIPTOR.message_types_by_name['AddBentoRequest'] = _ADDBENTOREQUEST
DESCRIPTOR.message_types_by_name['AddBentoResponse'] = _ADDBENTORESPONSE
DESCRIPTOR.message_types_by_name['UploadStatus'] = _UPLOADSTATUS
DESCRIPTOR.message_types_by_name['UpdateBentoRequest'] = _UPDATEBENTOREQUEST
DESCRIPTOR.message_types_by_name['UpdateBentoResponse'] = _UPDATEBENTORESPONSE
DESCRIPTOR.message_types_by_name['DangerouslyDeleteBentoRequest'] = _DANGEROUSLYDELETEBENTOREQUEST
DESCRIPTOR.message_types_by_name['DangerouslyDeleteBentoResponse'] = _DANGEROUSLYDELETEBENTORESPONSE
DESCRIPTOR.message_types_by_name['GetBentoRequest'] = _GETBENTOREQUEST
DESCRIPTOR.message_types_by_name['GetBentoResponse'] = _GETBENTORESPONSE
DESCRIPTOR.message_types_by_name['ListBentoRequest'] = _LISTBENTOREQUEST
DESCRIPTOR.message_types_by_name['ListBentoResponse'] = _LISTBENTORESPONSE
DESCRIPTOR.message_types_by_name['ContainerizeBentoRequest'] = _CONTAINERIZEBENTOREQUEST
DESCRIPTOR.message_types_by_name['ContainerizeBentoResponse'] = _CONTAINERIZEBENTORESPONSE
DESCRIPTOR.message_types_by_name['UploadBentoRequest'] = _UPLOADBENTOREQUEST
DESCRIPTOR.message_types_by_name['UploadBentoResponse'] = _UPLOADBENTORESPONSE
DESCRIPTOR.message_types_by_name['DownloadBentoRequest'] = _DOWNLOADBENTOREQUEST
DESCRIPTOR.message_types_by_name['DownloadBentoResponse'] = _DOWNLOADBENTORESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

BentoUri = _reflection.GeneratedProtocolMessageType('BentoUri', (_message.Message,), {
  'DESCRIPTOR' : _BENTOURI,
  '__module__' : 'repository_pb2'
  # @@protoc_insertion_point(class_scope:bentoml.BentoUri)
  })
_sym_db.RegisterMessage(BentoUri)

BentoServiceMetadata = _reflection.GeneratedProtocolMessageType('BentoServiceMetadata', (_message.Message,), {

  'BentoServiceEnv' : _reflection.GeneratedProtocolMessageType('BentoServiceEnv', (_message.Message,), {
    'DESCRIPTOR' : _BENTOSERVICEMETADATA_BENTOSERVICEENV,
    '__module__' : 'repository_pb2'
    # @@protoc_insertion_point(class_scope:bentoml.BentoServiceMetadata.BentoServiceEnv)
    })
  ,

  'BentoArtifact' : _reflection.GeneratedProtocolMessageType('BentoArtifact', (_message.Message,), {
    'DESCRIPTOR' : _BENTOSERVICEMETADATA_BENTOARTIFACT,
    '__module__' : 'repository_pb2'
    # @@protoc_insertion_point(class_scope:bentoml.BentoServiceMetadata.BentoArtifact)
    })
  ,

  'BentoServiceApi' : _reflection.GeneratedProtocolMessageType('BentoServiceApi', (_message.Message,), {
    'DESCRIPTOR' : _BENTOSERVICEMETADATA_BENTOSERVICEAPI,
    '__module__' : 'repository_pb2'
    # @@protoc_insertion_point(class_scope:bentoml.BentoServiceMetadata.BentoServiceApi)
    })
  ,

  'LabelsEntry' : _reflection.GeneratedProtocolMessageType('LabelsEntry', (_message.Message,), {
    'DESCRIPTOR' : _BENTOSERVICEMETADATA_LABELSENTRY,
    '__module__' : 'repository_pb2'
    # @@protoc_insertion_point(class_scope:bentoml.BentoServiceMetadata.LabelsEntry)
    })
  ,
  'DESCRIPTOR' : _BENTOSERVICEMETADATA,
  '__module__' : 'repository_pb2'
  # @@protoc_insertion_point(class_scope:bentoml.BentoServiceMetadata)
  })
_sym_db.RegisterMessage(BentoServiceMetadata)
_sym_db.RegisterMessage(BentoServiceMetadata.BentoServiceEnv)
_sym_db.RegisterMessage(BentoServiceMetadata.BentoArtifact)
_sym_db.RegisterMessage(BentoServiceMetadata.BentoServiceApi)
_sym_db.RegisterMessage(BentoServiceMetadata.LabelsEntry)

Bento = _reflection.GeneratedProtocolMessageType('Bento', (_message.Message,), {
  'DESCRIPTOR' : _BENTO,
  '__module__' : 'repository_pb2'
  # @@protoc_insertion_point(class_scope:bentoml.Bento)
  })
_sym_db.RegisterMessage(Bento)

AddBentoRequest = _reflection.GeneratedProtocolMessageType('AddBentoRequest', (_message.Message,), {
  'DESCRIPTOR' : _ADDBENTOREQUEST,
  '__module__' : 'repository_pb2'
  # @@protoc_insertion_point(class_scope:bentoml.AddBentoRequest)
  })
_sym_db.RegisterMessage(AddBentoRequest)

AddBentoResponse = _reflection.GeneratedProtocolMessageType('AddBentoResponse', (_message.Message,), {
  'DESCRIPTOR' : _ADDBENTORESPONSE,
  '__module__' : 'repository_pb2'
  # @@protoc_insertion_point(class_scope:bentoml.AddBentoResponse)
  })
_sym_db.RegisterMessage(AddBentoResponse)

UploadStatus = _reflection.GeneratedProtocolMessageType('UploadStatus', (_message.Message,), {
  'DESCRIPTOR' : _UPLOADSTATUS,
  '__module__' : 'repository_pb2'
  # @@protoc_insertion_point(class_scope:bentoml.UploadStatus)
  })
_sym_db.RegisterMessage(UploadStatus)

UpdateBentoRequest = _reflection.GeneratedProtocolMessageType('UpdateBentoRequest', (_message.Message,), {
  'DESCRIPTOR' : _UPDATEBENTOREQUEST,
  '__module__' : 'repository_pb2'
  # @@protoc_insertion_point(class_scope:bentoml.UpdateBentoRequest)
  })
_sym_db.RegisterMessage(UpdateBentoRequest)

UpdateBentoResponse = _reflection.GeneratedProtocolMessageType('UpdateBentoResponse', (_message.Message,), {
  'DESCRIPTOR' : _UPDATEBENTORESPONSE,
  '__module__' : 'repository_pb2'
  # @@protoc_insertion_point(class_scope:bentoml.UpdateBentoResponse)
  })
_sym_db.RegisterMessage(UpdateBentoResponse)

DangerouslyDeleteBentoRequest = _reflection.GeneratedProtocolMessageType('DangerouslyDeleteBentoRequest', (_message.Message,), {
  'DESCRIPTOR' : _DANGEROUSLYDELETEBENTOREQUEST,
  '__module__' : 'repository_pb2'
  # @@protoc_insertion_point(class_scope:bentoml.DangerouslyDeleteBentoRequest)
  })
_sym_db.RegisterMessage(DangerouslyDeleteBentoRequest)

DangerouslyDeleteBentoResponse = _reflection.GeneratedProtocolMessageType('DangerouslyDeleteBentoResponse', (_message.Message,), {
  'DESCRIPTOR' : _DANGEROUSLYDELETEBENTORESPONSE,
  '__module__' : 'repository_pb2'
  # @@protoc_insertion_point(class_scope:bentoml.DangerouslyDeleteBentoResponse)
  })
_sym_db.RegisterMessage(DangerouslyDeleteBentoResponse)

GetBentoRequest = _reflection.GeneratedProtocolMessageType('GetBentoRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETBENTOREQUEST,
  '__module__' : 'repository_pb2'
  # @@protoc_insertion_point(class_scope:bentoml.GetBentoRequest)
  })
_sym_db.RegisterMessage(GetBentoRequest)

GetBentoResponse = _reflection.GeneratedProtocolMessageType('GetBentoResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETBENTORESPONSE,
  '__module__' : 'repository_pb2'
  # @@protoc_insertion_point(class_scope:bentoml.GetBentoResponse)
  })
_sym_db.RegisterMessage(GetBentoResponse)

ListBentoRequest = _reflection.GeneratedProtocolMessageType('ListBentoRequest', (_message.Message,), {
  'DESCRIPTOR' : _LISTBENTOREQUEST,
  '__module__' : 'repository_pb2'
  # @@protoc_insertion_point(class_scope:bentoml.ListBentoRequest)
  })
_sym_db.RegisterMessage(ListBentoRequest)

ListBentoResponse = _reflection.GeneratedProtocolMessageType('ListBentoResponse', (_message.Message,), {
  'DESCRIPTOR' : _LISTBENTORESPONSE,
  '__module__' : 'repository_pb2'
  # @@protoc_insertion_point(class_scope:bentoml.ListBentoResponse)
  })
_sym_db.RegisterMessage(ListBentoResponse)

ContainerizeBentoRequest = _reflection.GeneratedProtocolMessageType('ContainerizeBentoRequest', (_message.Message,), {

  'BuildArgsEntry' : _reflection.GeneratedProtocolMessageType('BuildArgsEntry', (_message.Message,), {
    'DESCRIPTOR' : _CONTAINERIZEBENTOREQUEST_BUILDARGSENTRY,
    '__module__' : 'repository_pb2'
    # @@protoc_insertion_point(class_scope:bentoml.ContainerizeBentoRequest.BuildArgsEntry)
    })
  ,
  'DESCRIPTOR' : _CONTAINERIZEBENTOREQUEST,
  '__module__' : 'repository_pb2'
  # @@protoc_insertion_point(class_scope:bentoml.ContainerizeBentoRequest)
  })
_sym_db.RegisterMessage(ContainerizeBentoRequest)
_sym_db.RegisterMessage(ContainerizeBentoRequest.BuildArgsEntry)

ContainerizeBentoResponse = _reflection.GeneratedProtocolMessageType('ContainerizeBentoResponse', (_message.Message,), {
  'DESCRIPTOR' : _CONTAINERIZEBENTORESPONSE,
  '__module__' : 'repository_pb2'
  # @@protoc_insertion_point(class_scope:bentoml.ContainerizeBentoResponse)
  })
_sym_db.RegisterMessage(ContainerizeBentoResponse)

UploadBentoRequest = _reflection.GeneratedProtocolMessageType('UploadBentoRequest', (_message.Message,), {
  'DESCRIPTOR' : _UPLOADBENTOREQUEST,
  '__module__' : 'repository_pb2'
  # @@protoc_insertion_point(class_scope:bentoml.UploadBentoRequest)
  })
_sym_db.RegisterMessage(UploadBentoRequest)

UploadBentoResponse = _reflection.GeneratedProtocolMessageType('UploadBentoResponse', (_message.Message,), {
  'DESCRIPTOR' : _UPLOADBENTORESPONSE,
  '__module__' : 'repository_pb2'
  # @@protoc_insertion_point(class_scope:bentoml.UploadBentoResponse)
  })
_sym_db.RegisterMessage(UploadBentoResponse)

DownloadBentoRequest = _reflection.GeneratedProtocolMessageType('DownloadBentoRequest', (_message.Message,), {
  'DESCRIPTOR' : _DOWNLOADBENTOREQUEST,
  '__module__' : 'repository_pb2'
  # @@protoc_insertion_point(class_scope:bentoml.DownloadBentoRequest)
  })
_sym_db.RegisterMessage(DownloadBentoRequest)

DownloadBentoResponse = _reflection.GeneratedProtocolMessageType('DownloadBentoResponse', (_message.Message,), {
  'DESCRIPTOR' : _DOWNLOADBENTORESPONSE,
  '__module__' : 'repository_pb2'
  # @@protoc_insertion_point(class_scope:bentoml.DownloadBentoResponse)
  })
_sym_db.RegisterMessage(DownloadBentoResponse)


_BENTOSERVICEMETADATA_LABELSENTRY._options = None
_CONTAINERIZEBENTOREQUEST_BUILDARGSENTRY._options = None
# @@protoc_insertion_point(module_scope)



# Copyright 2019 Atalaya Tech, Inc.

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

# http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os

from bentoml.saved_bundle import load_from_dir
from bentoml.yatai.deployment.sagemaker.model_server import BentomlSagemakerServer

api_name = os.environ.get('API_NAME', None)
model_service = load_from_dir('/bento')
server = BentomlSagemakerServer(model_service, api_name)
app = server.app



# Copyright 2021 Atalaya Tech, Inc.

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

# http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from functools import lru_cache
import logging
from typing import Optional

from simple_di import Provide, inject

from bentoml.configuration.containers import BentoMLContainer


logger = logging.getLogger(__name__)


@inject
@lru_cache(maxsize=1)
def get_tracer(
    tracer_type=Provide[BentoMLContainer.config.tracing.type],
    zipkin_server_url: Optional[str] = Provide[
        BentoMLContainer.config.tracing.zipkin.url
    ],
    jaeger_server_address: Optional[str] = Provide[
        BentoMLContainer.config.tracing.jaeger.address
    ],
    jaeger_server_port: Optional[str] = Provide[
        BentoMLContainer.config.tracing.jaeger.port
    ],
):
    # isinstance check here allow trace to be used where the top-level entry point has
    # not yet implemented the wiring of BentoML config
    # TODO: remove this check after PR1543 https://github.com/bentoml/BentoML/pull/1543
    # if isinstance(tracer_type, Provider):
    # tracer_type = None
    # if isinstance(zipkin_server_url, Provider):
    # zipkin_server_url = None
    # if isinstance(jaeger_server_address, Provider):
    # jaeger_server_address = None
    # if isinstance(jaeger_server_port, Provider):
    # jaeger_server_port = None

    if tracer_type and tracer_type.lower() == 'zipkin' and zipkin_server_url:
        from bentoml.tracing.zipkin import get_zipkin_tracer

        logger.debug(
            "Initializing global zipkin tracer for collector endpoint: "
            f"{zipkin_server_url}"
        )
        return get_zipkin_tracer(zipkin_server_url)
    elif (
        tracer_type
        and tracer_type == 'jaeger'
        and jaeger_server_address
        and jaeger_server_port
    ):
        from bentoml.tracing.jaeger import get_jaeger_tracer

        logger.debug(
            "Initializing global jaeger tracer for opentracing server at "
            f"{jaeger_server_address}:{jaeger_server_port}"
        )
        return get_jaeger_tracer(jaeger_server_address, jaeger_server_port)
    else:
        from bentoml.tracing.noop import NoopTracer

        logger.debug("Tracing is disabled. Initializing no-op tracer")
        return NoopTracer()



# Copyright 2019 Atalaya Tech, Inc.

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

# http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging
import datetime

from sqlalchemy import (
    Column,
    Enum,
    String,
    Integer,
    JSON,
    Boolean,
    DateTime,
    UniqueConstraint,
    desc,
)
from sqlalchemy.orm.exc import NoResultFound
from google.protobuf.json_format import ParseDict

from bentoml.utils import ProtoMessageToDict
from bentoml.exceptions import YataiRepositoryException
from bentoml.yatai.db import Base
from bentoml.yatai.db.stores.label import (
    LabelStore,
    RESOURCE_TYPE,
)
from bentoml.yatai.proto.repository_pb2 import (
    UploadStatus,
    BentoUri,
    BentoServiceMetadata,
    Bento as BentoPB,
    ListBentoRequest,
)

logger = logging.getLogger(__name__)

DEFAULT_UPLOAD_STATUS = UploadStatus(status=UploadStatus.UNINITIALIZED)
DEFAULT_LIST_LIMIT = 40


class Bento(Base):
    __tablename__ = 'bentos'
    __table_args__ = tuple(UniqueConstraint('name', 'version', name='_name_version_uc'))

    id = Column(Integer, primary_key=True)
    name = Column(String, nullable=False)
    version = Column(String, nullable=False)

    # Storage URI for this Bento
    uri = Column(String, nullable=False)

    # Name is required for PostgreSQL and any future supported database which
    # requires an explicitly named type, or an explicitly named constraint in order to
    # generate the type and/or a table that uses it.
    uri_type = Column(
        Enum(*BentoUri.StorageType.keys(), name='uri_type'), default=BentoUri.UNSET
    )

    # JSON filed mapping directly to BentoServiceMetadata proto message
    bento_service_metadata = Column(JSON, nullable=False, default={})

    # Time of AddBento call, the time of Bento creation can be found in metadata field
    created_at = Column(DateTime, default=datetime.datetime.utcnow)

    # latest upload status, JSON message also includes last update timestamp
    upload_status = Column(
        JSON, nullable=False, default=ProtoMessageToDict(DEFAULT_UPLOAD_STATUS)
    )

    # mark as deleted
    deleted = Column(Boolean, default=False)


def _bento_orm_obj_to_pb(bento_obj, labels=None):
    # Backwards compatible support loading saved bundle created before 0.8.0
    if (
        'apis' in bento_obj.bento_service_metadata
        and bento_obj.bento_service_metadata['apis']
    ):
        for api in bento_obj.bento_service_metadata['apis']:
            if 'handler_type' in api:
                api['input_type'] = api['handler_type']
                del api['handler_type']
            if 'handler_config' in api:
                api['input_config'] = api['handler_config']
                del api['handler_config']
            if 'output_type' not in api:
                api['output_type'] = 'DefaultOutput'

    bento_service_metadata_pb = ParseDict(
        bento_obj.bento_service_metadata, BentoServiceMetadata()
    )
    bento_uri = BentoUri(
        uri=bento_obj.uri, type=BentoUri.StorageType.Value(bento_obj.uri_type)
    )
    if not bento_obj.upload_status:
        upload_status = DEFAULT_UPLOAD_STATUS
    else:
        upload_status = UploadStatus(
            status=UploadStatus.Status.Value(bento_obj.upload_status['status'])
        )
    if labels is not None:
        bento_service_metadata_pb.labels.update(labels)
    return BentoPB(
        name=bento_obj.name,
        version=bento_obj.version,
        uri=bento_uri,
        bento_service_metadata=bento_service_metadata_pb,
        status=upload_status,
    )


class MetadataStore(object):
    @staticmethod
    def add(sess, bento_name, bento_version, uri, uri_type):
        bento_obj = Bento()
        bento_obj.name = bento_name
        bento_obj.version = bento_version
        bento_obj.uri = uri
        bento_obj.uri_type = BentoUri.StorageType.Name(uri_type)
        return sess.add(bento_obj)

    @staticmethod
    def _get_latest(sess, bento_name):
        query = (
            sess.query(Bento)
            .filter_by(name=bento_name, deleted=False)
            .order_by(desc(Bento.created_at))
            .limit(1)
        )

        query_result = query.all()
        if len(query_result) == 1:
            labels = LabelStore.get(sess, RESOURCE_TYPE.bento, query_result[0].id)
            return _bento_orm_obj_to_pb(query_result[0], labels)
        else:
            return None

    @staticmethod
    def get(sess, bento_name, bento_version="latest"):
        if bento_version.lower() == "latest":
            return MetadataStore._get_latest(sess, bento_name)

        try:
            bento_obj = (
                sess.query(Bento)
                .filter_by(name=bento_name, version=bento_version, deleted=False)
                .one()
            )
            if bento_obj.deleted:
                # bento has been marked as deleted
                return None
            labels = LabelStore.get(sess, RESOURCE_TYPE.bento, bento_obj.id)
            return _bento_orm_obj_to_pb(bento_obj, labels)
        except NoResultFound:
            return None

    @staticmethod
    def update(sess, bento_name, bento_version, bento_service_metadata_pb):
        try:
            bento_obj = (
                sess.query(Bento)
                .filter_by(name=bento_name, version=bento_version, deleted=False)
                .one()
            )
            service_metadata = ProtoMessageToDict(bento_service_metadata_pb)
            bento_obj.bento_service_metadata = service_metadata
            if service_metadata.get('labels', None) is not None:
                bento = (
                    sess.query(Bento)
                    .filter_by(name=bento_name, version=bento_version)
                    .one()
                )
                LabelStore.add_or_update(
                    sess, RESOURCE_TYPE.bento, bento.id, service_metadata['labels']
                )
        except NoResultFound:
            raise YataiRepositoryException(
                "Bento %s:%s is not found in repository" % bento_name, bento_version
            )

    @staticmethod
    def update_upload_status(sess, bento_name, bento_version, upload_status_pb):
        try:
            bento_obj = (
                sess.query(Bento)
                .filter_by(name=bento_name, version=bento_version, deleted=False)
                .one()
            )
            # TODO:
            # if bento_obj.upload_status and bento_obj.upload_status.updated_at >
            # upload_status_pb.updated_at, update should be ignored
            bento_obj.upload_status = ProtoMessageToDict(upload_status_pb)
        except NoResultFound:
            raise YataiRepositoryException(
                "Bento %s:%s is not found in repository" % bento_name, bento_version
            )

    @staticmethod
    def dangerously_delete(sess, bento_name, bento_version):
        try:
            bento_obj = (
                sess.query(Bento)
                .filter_by(name=bento_name, version=bento_version, deleted=False)
                .one()
            )
            if bento_obj.deleted:
                raise YataiRepositoryException(
                    "Bento {}:{} has already been deleted".format(
                        bento_name, bento_version
                    )
                )
            bento_obj.deleted = True
        except NoResultFound:
            raise YataiRepositoryException(
                "Bento %s:%s is not found in repository" % bento_name, bento_version
            )

    @staticmethod
    def list(
        sess,
        bento_name=None,
        offset=None,
        limit=None,
        label_selectors=None,
        order_by=ListBentoRequest.created_at,
        ascending_order=False,
    ):
        query = sess.query(Bento)
        order_by = ListBentoRequest.SORTABLE_COLUMN.Name(order_by)
        order_by_field = getattr(Bento, order_by)
        order_by_action = order_by_field if ascending_order else desc(order_by_field)
        query = query.order_by(order_by_action)
        if bento_name:
            # filter_by apply filtering criterion to a copy of the query
            query = query.filter_by(name=bento_name)
        query = query.filter_by(deleted=False)
        if label_selectors is not None and (
            label_selectors.match_labels or label_selectors.match_expressions
        ):
            bento_ids = LabelStore.filter_query(
                sess, RESOURCE_TYPE.bento, label_selectors
            )
            query = query.filter(Bento.id.in_(bento_ids))

        # We are not defaulting limit to 200 in the signature,
        # because protobuf will pass 0 as value
        limit = limit or DEFAULT_LIST_LIMIT
        # Limit and offset need to be called after order_by filter/filter_by is
        # called
        query = query.limit(limit)
        if offset:
            query = query.offset(offset)

        query_result = query.all()
        bento_ids = [bento_obj.id for bento_obj in query_result]
        labels = LabelStore.list(sess, RESOURCE_TYPE.bento, bento_ids)
        result = [
            _bento_orm_obj_to_pb(bento_obj, labels.get(str(bento_obj.id)))
            for bento_obj in query_result
        ]
        return result



# Copyright 2019 Atalaya Tech, Inc.

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

# http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


BENTO_SERVICE_BUNDLE_SETUP_PY_TEMPLATE = """\
import setuptools
try:
    # for pip >= 10
    from pip._internal.req import parse_requirements
    try:
        # for pip >= 20.0
        from pip._internal.network.session import PipSession
    except ModuleNotFoundError:
        # for pip >= 10, < 20.0
        from pip._internal.download import PipSession
except ImportError:
    # for pip <= 9.0.3
    from pip.req import parse_requirements
    from pip.download import PipSession

try:
    raw = parse_requirements('requirements.txt', session=PipSession())

    # pip >= 20.1 changed ParsedRequirement attribute from `req` to `requirement`
    install_reqs = []
    for i in raw:
        try:
            install_reqs.append(str(i.requirement))
        except AttributeError:
            install_reqs.append(str(i.req))
except Exception:
    install_reqs = []

setuptools.setup(
    name='{name}',
    version='{pypi_package_version}',
    description="BentoML generated model module",
    long_description=\"\"\"{long_description}\"\"\",
    long_description_content_type="text/markdown",
    url="https://github.com/bentoml/BentoML",
    packages=setuptools.find_packages(),
    install_requires=install_reqs,
    include_package_data=True,
    package_data={{
        '{name}': ['bentoml.yml', 'artifacts/*']
    }},
    entry_points={{
        'console_scripts': [
            '{name}={name}:cli',
        ],
    }}
)
"""

MANIFEST_IN_TEMPLATE = """\
include {service_name}/bentoml.yml
graft {service_name}/artifacts
"""

MODEL_SERVER_DOCKERFILE_CPU = """\
FROM {docker_base_image}

# Configure PIP install arguments, e.g. --index-url, --trusted-url, --extra-index-url
ARG EXTRA_PIP_INSTALL_ARGS=
ENV EXTRA_PIP_INSTALL_ARGS $EXTRA_PIP_INSTALL_ARGS

ARG UID=1034
ARG GID=1034
RUN groupadd -g $GID -o bentoml && useradd -m -u $UID -g $GID -o -r bentoml

ARG BUNDLE_PATH=/home/bentoml/bundle
ENV BUNDLE_PATH=$BUNDLE_PATH
ENV BENTOML_HOME=/home/bentoml/

RUN mkdir $BUNDLE_PATH && chown bentoml:bentoml $BUNDLE_PATH -R
WORKDIR $BUNDLE_PATH

# copy over the init script; copy over entrypoint scripts
COPY --chown=bentoml:bentoml bentoml-init.sh docker-entrypoint.sh ./
RUN chmod +x ./bentoml-init.sh

# Copy docker-entrypoint.sh again, because setup.sh might not exist. This prevent COPY command from failing.
COPY --chown=bentoml:bentoml docker-entrypoint.sh setup.s[h] ./
RUN ./bentoml-init.sh custom_setup

COPY --chown=bentoml:bentoml docker-entrypoint.sh python_versio[n] ./
RUN ./bentoml-init.sh ensure_python

COPY --chown=bentoml:bentoml environment.yml ./
RUN ./bentoml-init.sh restore_conda_env

COPY --chown=bentoml:bentoml requirements.txt ./
RUN ./bentoml-init.sh install_pip_packages

COPY --chown=bentoml:bentoml docker-entrypoint.sh bundled_pip_dependencie[s]  ./bundled_pip_dependencies/
RUN rm ./bundled_pip_dependencies/docker-entrypoint.sh && ./bentoml-init.sh install_bundled_pip_packages

# copy over model files
COPY --chown=bentoml:bentoml . ./

# Default port for BentoML Service
EXPOSE 5000

USER bentoml
RUN chmod +x ./docker-entrypoint.sh
ENTRYPOINT [ "./docker-entrypoint.sh" ]
CMD ["bentoml", "serve-gunicorn", "./"]
"""  # noqa: E501

INIT_PY_TEMPLATE = """\
import os
import sys
import logging

from bentoml import saved_bundle, configure_logging
from bentoml.cli.bento_service import create_bento_service_cli

# By default, ignore warnings when loading BentoService installed as PyPI distribution
# CLI will change back to default log level in config(info), and by adding --quiet or
# --verbose CLI option, user can change the CLI output behavior
configure_logging(logging_level=logging.ERROR)

__VERSION__ = "{pypi_package_version}"

__module_path = os.path.abspath(os.path.dirname(__file__))

{service_name} = saved_bundle.load_bento_service_class(__module_path)

cli=create_bento_service_cli(__module_path)


def load():
    return saved_bundle.load_from_dir(__module_path)


__all__ = ['__version__', '{service_name}', 'load']
"""



# Copyright 2019 Atalaya Tech, Inc.

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

# http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging

from simple_di import Provide, inject

from bentoml.configuration.containers import BentoMLContainer
from bentoml.exceptions import YataiRepositoryException
from bentoml.yatai.proto.repository_pb2 import BentoUri
from bentoml.yatai.repository.base_repository import BaseRepository

logger = logging.getLogger(__name__)


class S3Repository(BaseRepository):
    @inject
    def __init__(
        self,
        base_url,
        endpoint_url: str = Provide[
            BentoMLContainer.config.yatai.repository.s3.endpoint_url
        ],
        signature_version: str = Provide[
            BentoMLContainer.config.yatai.repository.s3.signature_version
        ],
        expiration: int = Provide[
            BentoMLContainer.config.yatai.repository.s3.expiration
        ],
    ):
        import boto3
        from urllib.parse import urlparse

        self.uri_type = BentoUri.S3

        parse_result = urlparse(base_url)
        self.bucket = parse_result.netloc
        self.base_path = parse_result.path.lstrip('/')

        s3_client_args = {}
        s3_client_args['config'] = boto3.session.Config(
            signature_version=signature_version
        )
        if endpoint_url is not None:
            s3_client_args['endpoint_url'] = endpoint_url
        self.s3_client = boto3.client("s3", **s3_client_args)
        self.expiration = expiration

    def _get_object_name(self, bento_name, bento_version):
        if self.base_path:
            return "/".join([self.base_path, bento_name, bento_version]) + '.tar.gz'
        else:
            return "/".join([bento_name, bento_version]) + '.tar.gz'

    def add(self, bento_name, bento_version):
        # Generate pre-signed s3 path for upload

        object_name = self._get_object_name(bento_name, bento_version)
        try:
            response = self.s3_client.generate_presigned_url(
                'put_object',
                Params={'Bucket': self.bucket, 'Key': object_name},
                ExpiresIn=self.expiration,
            )
        except Exception as e:
            raise YataiRepositoryException(
                "Not able to get pre-signed URL on S3. Error: {}".format(e)
            )

        return BentoUri(
            type=self.uri_type,
            uri='s3://{}/{}'.format(self.bucket, object_name),
            s3_presigned_url=response,
        )

    def get(self, bento_name, bento_version):
        # Return s3 path containing uploaded Bento files

        object_name = self._get_object_name(bento_name, bento_version)

        try:
            response = self.s3_client.generate_presigned_url(
                'get_object',
                Params={'Bucket': self.bucket, 'Key': object_name},
                ExpiresIn=self.expiration,
            )
            return response
        except Exception:  # pylint: disable=broad-except
            logger.error(
                "Failed generating presigned URL for downloading saved bundle from s3,"
                "falling back to using s3 path and client side credential for"
                "downloading with boto3"
            )
            return 's3://{}/{}'.format(self.bucket, object_name)

    def dangerously_delete(self, bento_name, bento_version):
        # Remove s3 path containing related Bento files

        from botocore.exceptions import ClientError

        object_name = self._get_object_name(bento_name, bento_version)

        try:
            response = self.s3_client.delete_object(Bucket=self.bucket, Key=object_name)
            DELETE_MARKER = 'DeleteMarker'  # whether object is successfully deleted.

            # Note: as of boto3 v1.13.13. delete_object returns an incorrect format as
            # expected from documentation.
            # Expected format:
            # {
            #   'DeleteMarker': True|False,
            #   'VersionId': 'string',
            #   'RequestCharged': 'requester'
            # }
            # Current return:
            # {
            #   'ResponseMetadata': {
            #     'RequestId': '****************',
            #     'HostId': '*****/******',
            #     'HTTPStatusCode': 204,
            #     'HTTPHeaders': {
            #       'x-amz-id-2': '*****/xxxxx',
            #       'x-amz-request-id': '332EE9F7AB555D2B',
            #        'date': 'Tue, 19 May 2020 19:46:57 GMT',
            #        'server': 'AmazonS3'
            #     },
            #     'RetryAttempts': 0
            #   }
            # }
            # An open issue on github: https://github.com/boto/boto3/issues/759
            if DELETE_MARKER in response:
                if response[DELETE_MARKER]:
                    return
                else:
                    logger.warning(
                        f"BentoML has deleted service '{bento_name}:{bento_version}' "
                        f"from YataiService records, but it failed to delete the saved "
                        f"bundle files stored in s3://{self.bucket}/{object_name}, "
                        f"the files may have already been deleted by the user."
                    )
                    return
            elif 'ResponseMetadata' in response:
                # Note: Use head_object to 'check' is the object deleted or not.
                # head_object only try to retrieve the metadata without returning
                # the object itself.
                try:
                    self.s3_client.head_object(Bucket=self.bucket, Key=object_name)
                    logger.warning(
                        f"BentoML has deleted service '{bento_name}:{bento_version}' "
                        f"from YataiService records, but it failed to delete the saved "
                        f"bundle files stored in s3://{self.bucket}/{object_name}, "
                        f"the files may have already been deleted by the user."
                    )
                except ClientError as e:
                    # expected ClientError with Code 404, as target object should be
                    # deleted and 'head_object' should raise
                    error_response = e.response.get('Error', {})
                    error_code = error_response.get('Code', None)
                    if error_code == '404':
                        # Error code 404 means target file object does not exist, as
                        # expected after delete_object call
                        return
                    else:
                        # unexpected boto3 ClientError
                        raise e
            else:
                raise YataiRepositoryException(
                    'Unrecognized response format from s3 delete_object'
                )
        except Exception as e:
            raise YataiRepositoryException(
                "Not able to delete object on S3. Error: {}".format(e)
            )



# Copyright 2019 Atalaya Tech, Inc.

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

# http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import sys
import logging
import json

try:
    import download_extra_resources

    print("Downloading extra requirements and files from s3..")
    download_extra_resources.download_extra_resources()
    print("Finished downloading extra requirements and files")
except ImportError:
    # When function doesn't have extra resources or dependencies, we will not include
    # unzip_extra_resources and that will result with ImportError.  We will let it fail
    # silently
    pass

# After lambda "warm up" instance, the PYTHONPATH resets. Since the `/tmp/requirements`
# directory is already downloaded/extracted, we won't have a chance to add it to
# PYTHONPATH. We are going to ensure the requirements directory is in the PYTHONPATH, by
# checking it in the app.py
if '/tmp/requirements' not in sys.path:
    sys.path.append('/tmp/requirements')

# Set BENTOML_HOME to /tmp directory due to AWS lambda disk access restrictions
os.environ['BENTOML_HOME'] = '/tmp/bentoml/'
from bentoml.saved_bundle import load_from_dir  # noqa

logger = logging.getLogger('bentoml.lambda_app')

bento_name = os.environ['BENTOML_BENTO_SERVICE_NAME']
api_name = os.environ["BENTOML_API_NAME"]

bento_bundle_path = os.path.join('./', bento_name)
if not os.path.exists(bento_bundle_path):
    bento_bundle_path = os.path.join('/tmp/requirements', bento_name)

logger.debug('Loading BentoService bundle from path: "%s"', bento_bundle_path)
bento_service = load_from_dir(bento_bundle_path)
logger.debug('BentoService "%s" loaded successfully', bento_service.name)
bento_service_api = bento_service.get_inference_api(api_name)
logger.debug('BentoService API "%s" loaded successfully', {bento_service_api.name})

this_module = sys.modules[__name__]


def api_func(event, context):  # pylint: disable=unused-argument
    """
    Event – AWS Lambda uses this parameter to pass in event data to the handler. This
    parameter is usually of the Python dict type. It can also be list, str, int,
    float, or NoneType type. Currently BentoML only handles Lambda event coming from
    Application Load Balancer, in which case the parameter `event` must be type `dict`
    containing the HTTP request headers and body.

    You can find an example of which
    variables you can expect from the `event` object on the AWS Docs, here
    https://docs.aws.amazon.com/lambda/latest/dg/services-alb.html
    """

    if type(event) is dict and "headers" in event and "body" in event:
        prediction = bento_service_api.handle_aws_lambda_event(event)
        logger.info(
            json.dumps(
                {
                    'event': event,
                    'prediction': prediction["body"],
                    'status_code': prediction["statusCode"],
                }
            )
        )

        if prediction["statusCode"] >= 400:
            logger.warning('Error when predicting. Check logs for more information.')

        return prediction
    else:
        error_msg = (
            'Error: Unexpected Lambda event data received. Currently BentoML lambda '
            'deployment can only handle event triggered by HTTP request from '
            'Application Load Balancer.'
        )
        logger.error(error_msg)
        raise RuntimeError(error_msg)


setattr(this_module, api_name, api_func)
